package p2pclient;

import java.rmi.Remote;

/***
 * Interfaccia delle funzioni RMI condivise dal server RMI C2C_server
 *
 * @see p2pclient.C2C_server
 */
public interface C2C_RMIFuncConstructor extends Remote {


    /**
     * Interfaccia della funzione che restituisce il file chiesto sottoforma di array di byte.
     * @param k ClientUsername
     * @param v ClientAuthentication
     * @param name FileName
     * @param path FilePath
     * @return byte[]: Contenuto del file
     * @throws java.rmi.RemoteException
     */
    byte[] getFile(String k, String v, String name, String path) throws java.rmi.RemoteException;

    /**
     *
     * Interfaccia della funzione che cancella il file indicato tramite name e path.
     * @param k ClientUsername
     * @param v ClientAuthentication
     * @param name FileName
     * @param path FilePath
     * @return FileDeleted, true se il file e' stato cancellato, false altrimenti
     * @throws java.rmi.RemoteException
     */
    boolean deleteFile(String k, String v, String name, String path) throws java.rmi.RemoteException;
    
    /**
     *
     * Interfaccia della funzione che riceve il file inviato.
     * @param k ClientUsername
     * @param v ClientAuthentication
     * @param name FileContent
     * @param content FileName
     * @return FilePosted, true se il file e' stato inviato correttamente, false altrimenti
     * @throws java.rmi.RemoteException
     */
    boolean sendFile(String k, String v, byte[] content, String name) throws java.rmi.RemoteException;

    /**
     *
     * Intrefaccia della funzione utilizzata da ServerRMI per inviare a C2C_server le coppie di identificazione
     * (k,v) autorizzate ad effetture operazioni sui file e da inserire nell'hashtable
     * @param myu user di questo client per la connessione al server
     * @param myp password di questo client per la connessione al server
     * @param authu k autorizzata
     * @param authp v aurotizzata
     * @throws java.rmi.RemoteException
     */
    void addAuthUser(String myu, String myp, String authu, String authp) throws java.rmi.RemoteException;

    /**
     * Interfaccia della funzione di test che non necessita di autorizzazione per verificare se il server e' attivo
     * @return String
     * @throws java.rmi.RemoteException
     */
    String serverOn() throws java.rmi.RemoteException;


}
