package p2pclient;

import java.rmi.Naming;
import java.io.File;
import java.io.FileOutputStream;
import java.io.DataOutputStream;
import java.io.FileInputStream;
import java.io.DataInputStream;
import javax.swing.JOptionPane;
import javax.swing.JFileChooser;

/**
 * Client RMI C2C_client che si interfaccia al server RMI C2C_server,
 * per utilizzare le funzioni sui file tra client e client
 * @see p2pclient.C2C_server
 */
public class C2C_client{
	
    /**
     * Variabile boolean per attivare il degug
     */
    public static boolean DBG = true;
    public static int deleted = -1;

/**
 * Main che a seconda del nome funzione passato come parametro,
 * richiama le funzioni messe a disposizione dal server RMI C2C_server
 *
 * @param args nome fuzione, nome file, path file, username, password
 * @throws java.rmi.RemoteException
 */
    @SuppressWarnings("static-access")
    public static void main(String[] args)  throws java.rmi.RemoteException{
    	
		// Controllo dei valori in ingresso
		int nofargs = 6;
        if(args.length<nofargs){
            if(DBG) p2putility.Util.logClient("Argument/s missing (must be "+nofargs+").");
        	return;
        }
        
        int i = 0;
        for(;i<nofargs;i++)
        	if(args[i].length()<1)
        		{
        			if(DBG) p2putility.Util.logClient("Argument "+i+" not valid.");
        			return;
        		}
        
        // Assegnamento dei valori in ingresso alle variabili relative
        i = -1;
		String server_ip 	= "rmi://"+args[++i]+"/C2C_server";
		String requestfunc  = args[++i];
		String name 		= args[++i];
		String path			= args[++i];
		String user			= args[++i];
		String key			= args[++i];
		
		if(DBG) p2putility.Util.logClient("Using " + server_ip);

		try {
            C2C_RMIFuncConstructor obj = (C2C_RMIFuncConstructor)Naming.lookup(server_ip);
            if(DBG) p2putility.Util.logClient("obj C2C_RMIFuncConstructor creato.");

            if(requestfunc.toLowerCase().contentEquals("serverOn".toLowerCase()))
            {
                    if(DBG) p2putility.Util.logClient(obj.serverOn());
            }
            else
            if(requestfunc.toLowerCase().contentEquals("getFile".toLowerCase()))
            {

            		JFileChooser f = new JFileChooser(".");
            		f.setSelectedFile(new File(name));
			    	int r = f.showSaveDialog(null);
			    	
					if (r != f.APPROVE_OPTION)
					{
						if(DBG) p2putility.Util.logClient("save aborted.");
						return;
					}
					
					
			    	File fs = f.getSelectedFile();
				
					if(fs.exists())
					{
						r = JOptionPane.showConfirmDialog(null,"Overwrite file?","File already exist",1,JOptionPane.YES_NO_OPTION);
						
						if(r!=JOptionPane.YES_OPTION)
						{
							if(DBG) p2putility.Util.logClient("save aborted (no overwrite)");
							return;
						}
					}
		            
                    byte remfc[] = obj.getFile(user, key, name, path);

                    
                    if(remfc==null){if(DBG) p2putility.Util.logClient("getFile remote error (null)."); return;}
                    
                    try
                    {                    
	                    DataOutputStream locfw = new DataOutputStream(new FileOutputStream(fs));
	                    locfw.write(remfc);
	                    locfw.close();
	                    if(DBG) p2putility.Util.logClient("File "+name+" recived.");
                    }
                    catch(Exception xe)
                    {
                    	if(DBG) p2putility.Util.logClient("File "+name+" not recived.\n"+xe.getMessage());
                    	return;
                    }

            }
            else
            if(requestfunc.toLowerCase().contentEquals("deleteFile".toLowerCase()))
            {
                    boolean ret = obj.deleteFile(user, key, name, path);
                    if(ret)
                        deleted = 1;
                    else
                        deleted = 0;
                    if(DBG)
                			if(ret)	p2putility.Util.logClient("File "+name+" deleted.");
                			else	p2putility.Util.logClient("File "+name+" not delete.");
            }
            else
            if(requestfunc.toLowerCase().contentEquals("sendFile".toLowerCase()))
            {
            	File f = new File(path+name);
            	
            	if(!f.exists()){
		    		if(DBG)	p2putility.Util.logClient("file does not exist.");
		    		return;
		    	}
		    	
		    	long size = f.length();
		    	long maxsize = p2putility.Config.r_filemaxsizetosend();
		    	if(size>maxsize){
		    		if(DBG)	p2putility.Util.logClient("file too big (size: "+size+" byte, max: "+maxsize+" byte).");
		    		return;
		    	}
		
		    	byte content[] = new byte[(int)size];
		    	
		    	try
		    	{
		    		DataInputStream fr = new DataInputStream(new FileInputStream(f));
		    		fr.read(content);
		            fr.close();
		    	}
		    	catch(Exception xe)
		    	{
		    		if(DBG) p2putility.Util.logClient("C2C_client err: " + xe.getMessage());
		    		return;
		    	}
		    
                boolean ret = obj.sendFile(user, key, content, name);
                
                if(DBG)
                   	if(ret)	p2putility.Util.logClient("File "+name+" sent.");
                	else	p2putility.Util.logClient("File "+name+" not send.");
                	
            }
            else
            	if(DBG) p2putility.Util.logClient("Function not implemented.");
            	
            	
        }
        catch (Exception e)
        {
			if(DBG) p2putility.Util.logClient("Client exception: "+e);
			return;
		}

    }

}
