package p2pclient;

import java.rmi.Naming;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;
import java.io.File;
import java.io.FileInputStream;
import java.io.DataInputStream;
import java.io.FileOutputStream;
import java.io.DataOutputStream;
import javax.swing.JOptionPane;
import javax.swing.JFileChooser;
import javax.swing.table.DefaultTableModel;


/**
 *
 * Classe che implementa il server RMI C2C_server e le funzioni messe a disposizione
 * dall'interfaccia C2C_RMIFuncConstructor
 * @see p2pclient.C2C_RMIFuncConstructor
 * @see p2pclient.C2C_client
 */
public class C2C_server extends UnicastRemoteObject implements C2C_RMIFuncConstructor{
    
    /**
     * Variabile boolean per attivare il degug
     */
    public static boolean DBG = true;
    static C2C_server obj;

 /**
  * Costruttore della classe C2C_server
  * @throws java.rmi.RemoteException
  */
    protected C2C_server() throws java.rmi.RemoteException{
        super();
    }

/**
 * Main che crea un'istanza di RMIRegistry se non e' gia' stata creata da ServerRMI
 * e registra le sue funzioni
 * @param args void
 * @throws java.rmi.RemoteException
 */
    public static void main(String[] args)  throws java.rmi.RemoteException{
        
        try
        {
			obj = new C2C_server();

            Registry registry;

            if(!p2putility.Config.r_serverOpen())
                registry = LocateRegistry.createRegistry(p2putility.Config.r_rmiregport());
            else
                registry = LocateRegistry.getRegistry();
            registry.rebind("C2C_server",obj);
            Naming.rebind("C2C_server", obj);

			if(DBG) p2putility.Util.logClient("C2C_server e' stato registrato");
		} 
		catch (Exception e)
		{
			if(DBG) p2putility.Util.logClient("C2C_server err: " + e);
		}
    }


/**
 * Metodo che controlla se (k,v) fornite dal client sono gia' presenti dell'hashtable
 *
 * @param k key d'accesso alle funzionalita'
 * @param v value d'accesso alle funzionalita'
 * @return true, se autorizzato, false, altrimenti
 */
    private boolean authorize(String k, String v)
    {
        if(DBG) p2putility.Util.logClient("Client SERVER SIDE: authorize("+k+","+v+")="+p2putility.Config.cauth);
    	try
    	{
    		if(v.length()>5)
	    		if(v.compareTo(p2putility.Config.cauth.get(k).toString())==0)
	    			return true;
	    }
	    catch(Exception e)
	    {
	    	return false;
	    }
	    
    	return false;
    }

/**
 * Funzione che ritorna il valore dell'host del client
 *
 * @return String: host del client
 */
    @SuppressWarnings("static-access")
    private String clientIP()
    {
    	try{
    		return obj.getClientHost();
    	}
    	catch(Exception e)
    	{
    		return "unknown";
    	}
    }


/**
 * Funzione di test che non necessita di autorizzazione per verificare se il server e' attivo
 * @return String
 */
    public String serverOn()
    {
        if(DBG) p2putility.Util.logClient("Client SERVER SIDE: ON! from: "+clientIP());
        return "CLIENT SIDE: SERVER ON!";
    }

/**
 * Funzione che restituisce il file chiesto sottoforma di array di byte.
 *
 * @param k ClientUsername
 * @param v ClientAuthentication
 * @param name FileName
 * @param path FilePath
 * @return byte[]: FileContent
*/
    public byte[] getFile(String k, String v, String name, String path)
    {
    	
    	if(DBG) p2putility.Util.logClient("Client SERVER SIDE: getFile("+path+","+name+") from: "+clientIP());
    	
    	// ****** INDISPENSABILE PER LE FUNZIONI CHE FANNO QUALCOSA DI PRATICO
    	// ****** Un utente qualsiasi potrebbe prendere e/o cancellare tutti i file del tuo PC
    	if(!authorize(k,v)){
    		if(DBG)	p2putility.Util.logClient("Client SERVER SIDE: client "+clientIP()+" logged as ("+k+","+v+") is not authorized.");
    		return null;
    	}
    	// END **************************************
    	File remf = new File(path+name);
    	
    	if(!remf.exists()){
    		if(DBG)	p2putility.Util.logClient("Client SERVER SIDE: file does not exist.");
    		return null;
    	}
    	
    	long size = remf.length();
    	long maxsize = p2putility.Config.r_filemaxsizetorecv();
    	if(size>maxsize){
    		if(DBG)	p2putility.Util.logClient("Client SERVER SIDE: file too big (size: "+size+" byte, max: "+maxsize+" byte).");
    		return null;
    	}

    	byte content[] = new byte[(int)size];
    	
    	try
    	{
    		DataInputStream remfr = new DataInputStream(new FileInputStream(remf));
    		remfr.read(content);
            remfr.close();
    	}
    	catch(Exception xe)
    	{
    		if(DBG) p2putility.Util.logClient("C2C_server err: " + xe);
    		return null;
    	}
    	
        return content;
    }

/**
 * Funzione che cancella il file indicato tramite name e path.
 *
 * @param k ClientUsername
 * @param v ClientAuthentication
 * @param name FileName
 * @param path FilePath
 * @return FileDeleted, true se il file e' stato cancellato, false altrimenti
*/
    public boolean deleteFile(String k, String v, String name, String path)
    {
        
        if(DBG) p2putility.Util.logClient("Client SERVER SIDE: deleteFile("+path+","+name+") from: "+clientIP());
    	
    	// ****** INDISPENSABILE PER LE FUNZIONI CHE FANNO QUALCOSA DI PRATICO
   		// ****** Un utente qualsiasi potrebbe prendere e/o cancellare tutti i file del tuo PC
    	if(!authorize(k,v)){
    		if(DBG)	p2putility.Util.logClient("Client SERVER SIDE: client "+clientIP()+" logged as ("+k+","+v+") is not authorized.");
    		return false;
    	}
    	
    	File remf = new File(path+name);
    	
    	if(!remf.exists()){
    		if(DBG)	p2putility.Util.logClient("Client SERVER SIDE: file "+name+" does not exist.");
    		return false;
    	}
        
    	try
    	{
            boolean ris = remf.delete();
            if(ris){
                DefaultTableModel model = (DefaultTableModel) p2pgui.ClientFrame.jtabshare.getModel();
                int numrow = model.getRowCount();
                for(int i=0; i< numrow; i++)
                    if(((model.getValueAt(i, 0)).toString().equals(name))&&((model.getValueAt(i, 1)).toString().equals(path))){
                        model.removeRow(i);
                        model.fireTableRowsDeleted(i, i);
                        break;
                    }
                p2putility.Config.cauth.remove(k);
                p2pgui.ClientFrame.h.remove(remf.getAbsolutePath());
            }
    	   	return ris;
    	}
    	catch(Exception xe){
    		if(DBG) p2putility.Util.logClient("C2C_server err: " + xe);
    		return false;
    	}
    	
    }
    
/**
 * Funzione che riceve il file inviato.
 *
 * @param k ClientUsername
 * @param v ClientAuthentication
 * @param name FileContent
 * @param content FileName
 * @return FilePosted, true se il file e' stato inviato correttamente, false altrimenti
*/
    @SuppressWarnings("static-access")
    public boolean sendFile(String k, String v, byte[] content, String name){
    	if(DBG) p2putility.Util.logClient("Client SERVER SIDE: sendFile("+name+") from: "+clientIP());
    	
    	// ****** INDISPENSABILE PER LE FUNZIONI CHE FANNO QUALCOSA DI PRATICO
   		// ****** Un utente qualsiasi potrebbe prendere e/o cancellare tutti i file del tuo PC
    	if(!authorize(k,v)){
    		if(DBG)	p2putility.Util.logClient("Client SERVER SIDE: client "+clientIP()+" logged as ("+k+","+v+") is not authorized.");
    		return false;
    	}
    	// *********************************************************************

        if(!p2putility.Config.open_upload_conn()) { // troppe connessioni attive
            if(DBG) p2putility.Util.logClient("Client SERVER SIDE: too much connection");
            return false; 
        }

    	// ************* FINE POLITICHE DI ACCETTAZIONE
    	
    	JFileChooser f = new JFileChooser(".");
    	f.setSelectedFile(new File(name));
    	//f.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
    	int r = f.showSaveDialog(null);
    	
		if (r != f.APPROVE_OPTION)
		{
			if(DBG) p2putility.Util.logClient("Client SERVER SIDE: rejected send");
			return false;
		}
		
		
    	File fs = f.getSelectedFile();
	
		if(fs.exists())
		{
			r = JOptionPane.showConfirmDialog(null,"Overwrite file?","File already exist",1,JOptionPane.YES_NO_OPTION);
			
			if(r!=JOptionPane.YES_OPTION)
			{
				if(DBG) p2putility.Util.logClient("Client SERVER SIDE: rejected send (no overwrite)");
				return false;
			}
		}

		
		try
        {                    
            DataOutputStream fw = new DataOutputStream(new FileOutputStream(fs));
            fw.write(content);
            fw.close();

            p2putility.Config.close_upload_conn(); // decrementa il numero di connessioni dei client
            
            return true;
        }
        catch(Exception xe)
        {
        	if(DBG) p2putility.Util.logClient("C2C_server err: " + xe);
    		return false;
        }

    }


/**
 * Funzione utilizzata da ServerRMI per inviare a C2C_server le coppie di identificazione
 * (k,v) autorizzate ad effetture operazioni sui file e da inserire nell'hashtable
 *
 * @param myu user di questo client per la connessione al server
 * @param myp password di questo client per la connessione al server
 * @param authu k autorizzata
 * @param authp v aurotizzata
 * @throws java.rmi.RemoteException
 */
    public void addAuthUser(String myu, String myp, String authu, String authp) throws java.rmi.RemoteException{
      if(p2pgui.ClientFrame.isMyUser(myu)){ // user e' una var privata! Funzione public boolean che la verifica.
        if(p2pgui.ClientFrame.isMyPswd(myp)) // pw e' una var privata! Funzione public boolean che la verifica.
        {
            if(!p2putility.Config.cauth.containsKey(authu))
                p2putility.Config.cauth.put(authu,authp);
            if(DBG) p2putility.Util.logClient("Client SERVER SIDE: Client "+authu+" accepted.");
                return;
        }

        if(DBG) p2putility.Util.logClient("Client SERVER SIDE: Connection refused! You are not my server.");
       }
    }

}
