package p2pclient;

import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.util.Hashtable;

/**
 * Classe che implementa un client mediante socket, che effettua in broadcast
 * la richiesta dei server attivi
 * @see p2pserver.ServerListener
 */
public class ClientGetLocalServerList{
    
/**
 * Costruttore della classe
 */
    public ClientGetLocalServerList(){super();} //costruttore

/**
 * Funzione che invia un Datagram in broadcast e che fino allo scadere del timeout
 * resta in attesa delle risposte dei server attivi
 *
 * @param bip broadcast ip
 * @param port porta su cui inviare la richiesta
 * @param timeout
 * @param dbg debug
 * @return Hastable contenente gli indirizzi ip dei server che hanno risposto
 * @see p2pserver.ServerListener
 */
    public static Hashtable start(String bip, int port, int timeout, boolean dbg){

        Hashtable input = new Hashtable();
        byte[] buf = new byte[256];

        DatagramSocket socket;
        try
        {
        	buf = InetAddress.getLocalHost().getHostAddress().getBytes(); // This IP
            socket = new DatagramSocket();
            InetAddress address= InetAddress.getByName(bip); // (Broadcast) address

            DatagramPacket packet = new DatagramPacket(buf, buf.length,address, port);
            socket.send(packet);

            do
            {
                buf = new byte[256];
                packet = new DatagramPacket(buf, buf.length);
            	socket.setSoTimeout(timeout); // Quando scatta provoca un eccezione (termina il while)
	            socket.receive(packet);

	            String s = new String(packet.getData());
                s = s.substring(0, s.indexOf(0));

			    if(isServerIp(s)){
		            if(!input.containsValue(s))
		            	input.put(input.size(),s);
                }
                else
                  if(dbg) p2putility.Util.logClient("Ricevuto IP non valido: "+s);
		    }
		    while(true);

        }
        catch (Exception ex){ /* Rileva anche il timeout, che non e' un errore */ }

        if(dbg) p2putility.Util.logClient("Lista definitiva dei server: "+input.toString()); // HashTable definitiva

        return input;
    }
    
/**
 * Funzione che verifica se la stringa passata come parametro e' un indirizzo ip
 * @param ins stringa da verificare
 * @return true se ins e' un ip, false altrimenti
 */

	private static boolean isServerIp(String ins)
  	{
  		try
  		{
			if(ins==null) return false;
            if(ins.length()<7) return false;

            int nofp = 0, i;
            for(i=0;i<ins.length();i++)
                if(ins.charAt(i)=='.')
                    nofp++;
            
            if(nofp!=3) return false;

            String tmps = ins.substring(0, ins.indexOf('.'));
            i = Integer.parseInt(tmps);
            if((i<0)||(i>255)) return false;
            
            ins = ins.substring(tmps.length()+1);

            tmps = ins.substring(0, ins.indexOf('.'));
            i = Integer.parseInt(tmps);
            if((i<0)||(i>255)) return false;

            ins = ins.substring(tmps.length()+1);
            
            tmps = ins.substring(0, ins.indexOf('.'));
            i = Integer.parseInt(tmps);
            if((i<0)||(i>255)) return false;

            ins = ins.substring(tmps.length()+1);
            
            i = Integer.parseInt(ins);
            if((i<0)||(i>255)) return false;

            /*REGEXP?!
              final Pattern IP_PATTERN =
              Pattern.compile("b(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?).)"
                                    + "{3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)b");
    		return IP_PATTERN.matcher(ins).matches();*/
    		return true;
  		}
  		catch(Exception e){ return false; }
  	} 
}
