package p2putility;

import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.util.Hashtable;

/**
 * Classe che permette la configurazione di tutto il programma
 * in base alle specifiche esigenze dell'utente, che, per fare cio',
 * agisce sul file di configurazione presente nella directory base del programma.
 */
public class Config {

    //Inizializzazione delle variabili configurabili da file
    private static String classpath = ""; 
    private static String rmicpath = "";
    private static String configpath = "";
    private static String userlistfile = "";
	private static String defip = "";
    private static String userp2p = "";
	private static String netmask = "";
	private static String dbp2p = "";
    private static String clientlog = "";
    private static String serverlog = "";
    private static String genericlog = "";

    private static long filemaxsizetosend = 0;
    private static long filemaxsizetorecv = 0;
    private static int port = 0;
    private static int availableconn = 0;
    private static int threadtimeout = 0;
    private static int rmiregport= 0;
    private static int bcasttimeout= 0;

     // Non configurabili da file
    private static int nofactive = availableconn;
    private static String currentpath = System.getProperty("user.dir"); // current application absolute path (es. c:\prg\)
    private static String configfile = "config.fil";
    private static boolean serverOpen = false;
    private static boolean clientOpen = false;
    private static String ico_server_enable = "/icons/enable_server_ico.png";
    private static String ico_server_disable = "/icons/no_connection_ico.png";
    /**
     * Hashtable contenente le coppie (k,v) fornite dal server, per autenticare
     * un client che vuole accedere ad un file su un altro client
     * @see p2pclient.C2C_server
     */
    public static Hashtable cauth = new Hashtable();

    /**
     * Costruttore.
     * Se non trova una configurazione valida nel file di configurazione
     * carica le impostazioni di default impostate dai programmatori.
     */
    public Config(){
        super();

        if(loadConfig()) return; //Se la configurazione UTENTE e' andata a buon fine esce

        // Altrimenti, se non carica la configurazione definibile dall'UTENTE applica i
        //DEFAULT SETTINGS
        classpath = "/build/classes/";
        rmicpath = "";
        configpath = "/p2pfile/";
        userlistfile = "userlist.csv";
        defip = "127.0.0.1";
        port = 9090;
        userp2p = "p2puser";
        dbp2p = "p2pdb";
        netmask = "255.255.255.0";
        availableconn = 1;
        nofactive = availableconn;
        threadtimeout = 10000;
        rmiregport= 1099;
        clientlog = "LOG_client.txt";
        serverlog = "LOG_server.txt";
        genericlog = "LOG_generic.txt";
        bcasttimeout = 3000;
        filemaxsizetosend = 1048576;//1Mb
        filemaxsizetorecv = 1048576;//1Mb
        //END DEFAULT SETTINGS

    }

    /**
     * Caricare la configurazione dal file apposito.
     * - In caso di successo (anche parziale) le variabili configurate
     * correttamente aranno impostate. Restituisce true.
     * - In caso di parziale insuccesso (sperimentale) scrive un avviso
     * sul file di LOG generico. Restituisce true.
     * In caso di insuccesso restituisce false.
     *
     * @return boolean false se vi e' stato un fallimento, true altrimenti
     *
     */
    private boolean loadConfig(){
        try{
            File f = new File(currentpath+File.separator+configfile);

            if(!f.exists()) return false;

            DataInputStream d = new DataInputStream(new FileInputStream(f));

            java.util.Hashtable total = new java.util.Hashtable();

            while(d.available()>0){

                // Ignoro i commenti e le righe inutili
                String v = "";
                do{
                    v = d.readLine();
                    if(v==null)
                        v = "#";
                }
                while((d.available()>0)&&((v.startsWith("#")||(v.length()<2))));

                String ids = "classpath=";
                if(v.startsWith(ids)){
                    classpath = v.substring(ids.length()).trim();
                    if(!total.containsKey(ids))
                        total.put(ids, 0);
                }
                else{ // 1
                    ids = "rmicpath=";
                    if(v.startsWith(ids)){
                        rmicpath = v.substring(ids.length()).trim();
                        if(!total.containsKey(ids))
                            total.put(ids, 1);
                    }
                else{ // 2
                    ids = "configpath=";
                    if(v.startsWith(ids)){
                        configpath = v.substring(ids.length()).trim();
                        if(!total.containsKey(ids))
                            total.put(ids, 2);
                    }
                else{ // 3
                    ids = "userlistfile=";
                    if(v.startsWith(ids)){
                        userlistfile = v.substring(ids.length()).trim();
                        if(!total.containsKey(ids))
                            total.put(ids, 3);
                    }
                else{ // 4
                    ids = "defip=";
                    if(v.startsWith(ids)){
                        defip = v.substring(ids.length()).trim();
                        if(!total.containsKey(ids))
                            total.put(ids, 4);
                    }
                else{ // 5
                    ids = "port=";
                    if(v.startsWith(ids)){
                        port = Integer.parseInt(v.substring(ids.length()).trim());
                        if(!total.containsKey(ids))
                            total.put(ids, 5);
                    }
                else{ // 6
                    ids = "userp2p=";
                    if(v.startsWith(ids)){
                        userp2p = v.substring(ids.length()).trim();
                        if(!total.containsKey(ids))
                            total.put(ids, 6);
                    }
                else{ // 7
                    ids = "dbp2p=";
                    if(v.startsWith(ids)){
                        dbp2p = v.substring(ids.length()).trim();
                        if(!total.containsKey(ids))
                            total.put(ids, 7);
                    }
                else{  // 8
                    ids = "netmask=";
                    if(v.startsWith(ids)){
                        netmask = v.substring(ids.length()).trim();
                        if(!total.containsKey(ids))
                            total.put(ids, 8);
                    }
                else{  // 9
                    ids = "availableconn=";
                    if(v.startsWith(ids)){
                        availableconn = Integer.parseInt(v.substring(ids.length()).trim());
                        nofactive = availableconn;
                        if(!total.containsKey(ids))
                            total.put(ids, 9);
                    }
                else{  // 10
                    ids = "threadtimeout=";
                    if(v.startsWith(ids)){
                        threadtimeout = Integer.parseInt(v.substring(ids.length()).trim());
                        if(!total.containsKey(ids))
                            total.put(ids, 10);
                    }
                else{  // 11
                    ids = "rmiregport=";
                    if(v.startsWith(ids)){
                        rmiregport = Integer.parseInt(v.substring(ids.length()).trim());
                        if(!total.containsKey(ids))
                            total.put(ids, 11);
                    }
                else{  // 12
                    ids = "clientlog=";
                    if(v.startsWith(ids)){
                         clientlog = v.substring(ids.length()).trim();
                         if(!total.containsKey(ids))
                            total.put(ids, 12);
                    }
                else{  // 13
                    ids = "serverlog=";
                    if(v.startsWith(ids)){
                         serverlog = v.substring(ids.length()).trim();
                         if(!total.containsKey(ids))
                            total.put(ids, 13);
                    }
                else{  // 14
                    ids = "genericlog=";
                    if(v.startsWith(ids)){
                         genericlog = v.substring(ids.length()).trim();
                         if(!total.containsKey(ids))
                            total.put(ids, 14);
                    }
                else{  // 15
                    ids = "bcasttimeout=";
                    if(v.startsWith(ids)){
                         bcasttimeout = Integer.parseInt(v.substring(ids.length()).trim());
                         if(!total.containsKey(ids))
                            total.put(ids, 15);
                    }
                else{  // 16
                    ids = "filemaxsizetorecv=";
                    if(v.startsWith(ids)){
                         filemaxsizetorecv = Integer.parseInt(v.substring(ids.length()).trim());
                         if(!total.containsKey(ids))
                            total.put(ids, 16);
                    }
                else{  // 17
                    ids = "filemaxsizetosend=";
                    if(v.startsWith(ids)){
                         filemaxsizetosend = Integer.parseInt(v.substring(ids.length()).trim());
                         if(!total.containsKey(ids))
                            total.put(ids, 17);
                    }
                }//17
                }//16
                }//15
                }//14
                }//13
                }//12
                }//11
                }//10
                }//9
                }//8
                }//7
                }//6
                }//5
                }//4
                }//3
                }//2
                }//1

            }
            d.close();

            if(total.size() != 18)
                p2putility.Util.logGeneric("Config file WARNING: missing arguments!\n"+total);

            return true;
        }
        catch(Exception x){
            p2putility.Util.logGeneric("Config file ERROR: "+x);
            return false;
        }
    }

    /** Permette la lettura della varibile relativa in modo sicuro
     * @return String variabile relativa (identificata dal nome successivo a 'r_')
    */
    public static String r_currentpath(){return currentpath;}

    /** Permette la lettura della varibile relativa in modo sicuro
     * @return String variabile relativa (identificata dal nome successivo a 'r_')
    */
    public static String r_classpath(){return classpath;}

    /** Permette la lettura della varibile relativa in modo sicuro
     * @return String variabile relativa (identificata dal nome successivo a 'r_')
    */
    public static String r_rmicpath(){return rmicpath;}

    /** Permette la lettura della varibile relativa in modo sicuro
     * @return String variabile relativa (identificata dal nome successivo a 'r_')
    */
    public static String r_configpath(){return configpath;}

    /** Permette la lettura della varibile relativa in modo sicuro
     * @return String variabile relativa (identificata dal nome successivo a 'r_')
    */
    public static String r_userlistfile(){return userlistfile;}

    /** Permette la lettura della varibile relativa in modo sicuro
     * @return int variabile relativa (identificata dal nome successivo a 'r_')
    */
    public static int r_port(){return port;}

    /** Permette la lettura della varibile relativa in modo sicuro
     * @return String variabile relativa (identificata dal nome successivo a 'r_')
    */
    public static String r_defip(){return defip;}

    /** Permette la lettura della varibile relativa in modo sicuro
     * @return String variabile relativa (identificata dal nome successivo a 'r_')
    */
    public static String r_userp2p(){return userp2p;}

    /** Permette la lettura della varibile relativa in modo sicuro
     * @return String variabile relativa (identificata dal nome successivo a 'r_')
    */
    public static String r_dbp2p(){return dbp2p;}

    /** Permette la lettura della varibile relativa in modo sicuro
     * @return String variabile relativa (identificata dal nome successivo a 'r_')
    */
    public static String r_netmask(){return netmask;}

    /** Permette la lettura della varibile relativa in modo sicuro
     * @return int variabile relativa (identificata dal nome successivo a 'r_')
    */
    public static int r_threadtimeout(){return threadtimeout;}

    /** Permette la lettura della varibile relativa in modo sicuro
     * @return int variabile relativa (identificata dal nome successivo a 'r_')
    */
    public static int r_rmiregport(){return rmiregport;}

    /** Permette la lettura della varibile relativa in modo sicuro
     * @return String percorso del file di log relativo
    */
    public static String r_clientlog(){return currentpath+configpath+clientlog;}

    /** Permette la lettura della varibile relativa in modo sicuro
     * @return String percorso del file di log relativo
    */
    public static String r_serverlog(){return currentpath+configpath+serverlog;}

    /** Permette la lettura della varibile relativa in modo sicuro
     * @return String percorso del file di log relativo
    */
    public static String r_genericlog(){return currentpath+configpath+genericlog;}

    /** Permette la lettura della varibile relativa in modo sicuro
     * @return int variabile relativa (identificata dal nome successivo a 'r_')
    */
    public static int r_bcasttimeout(){return bcasttimeout;}

    /** Permette la lettura della varibile relativa in modo sicuro
     * @return long massima dimensione del file da ricevere - Integer.MAX_VALUE e' il massimo accettabile (2Gb)
    */
    public static long r_filemaxsizetorecv(){
        if(filemaxsizetorecv>Integer.MAX_VALUE)
            return Integer.MAX_VALUE;
        return filemaxsizetorecv;
    }

    /** Permette la lettura della varibile relativa in modo sicuro
     * @return long massima dimensione del file da inviare - Integer.MAX_VALUE e' il massimo accettabile (2Gb)
    */
    public static long r_filemaxsizetosend(){
        if(filemaxsizetosend>Integer.MAX_VALUE)
            return Integer.MAX_VALUE;
        return filemaxsizetosend;
    }

    /** Permette la lettura della varibile relativa in modo sicuro
     * @return boolean true se e' possible aprire un'altra connessione
    */
    public static boolean open_upload_conn(){
        if(nofactive<0)
            return false;
        nofactive--;
        return true;
    }

    /** Permette la lettura della varibile relativa in modo sicuro
     * @return boolean true se e' possible chiudere un'altra connessione
    */
    public static boolean close_upload_conn(){
        if(nofactive>=availableconn)
            return false;
        nofactive++;
        return true;
    }

    // Funzioni per le varibili non configurabili da file

    /** Permette la lettura della varibile relativa in modo sicuro
     * @return boolean variabile relativa (identificata dal nome successivo a 'r_')
    */
    public static boolean r_serverOpen() {return serverOpen;}

    /** Permette la scrittura della varibile relativa in modo sicuro
     * @param value valore da inserire nella variabile relativa (identificata dal nome successivo a 'r_')
    */
    public static void w_serverOpen(boolean value) {serverOpen = value;}

    /** Permette la lettura della varibile relativa in modo sicuro
     * @return boolean variabile relativa (identificata dal nome successivo a 'r_')
    */
    public static boolean r_clientOpen() {return clientOpen;}
    
    /** Permette la scrittura della varibile relativa in modo sicuro
     * @param value valore da inserire nella variabile relativa (identificata dal nome successivo a 'r_')
    */
    public static void w_clientOpen(boolean value) {clientOpen = value;}

    /** Permette la lettura della varibile relativa in modo sicuro
     * @return boolean variabile relativa (identificata dal nome successivo a 'r_')
    */
    public static String r_ico_server_enable() {return ico_server_enable;}

    /** Permette la lettura della varibile relativa in modo sicuro
     * @return boolean variabile relativa (identificata dal nome successivo a 'r_')
    */
    public static String r_ico_server_disable() {return ico_server_disable;}

}
