
package p2pserver;

import java.sql.*;

/**
 * Classe che mette a disposizione funzioni per interfacciarsi al DBMS in
 * modo rapido ed eventualmente sicuro.
 */
public class Connecttodbms{

/**
 * Costruttore
 */
public Connecttodbms(){super();}

/**
 * Restituisce una connessione (Connection) al DB selezionato.
 * Lavora con DBMS MySQL in localhost. (vedi specifiche)
 *
 * @param speak attiva l'output di debug
 * @param dbms nome del DB a cui connettersi
 * @param user username dell'utente col quale connetteri
 * @param pswd password dell'utente col quale connetteri
 * @return Connection connessione al DBMS
 */
public static Connection connect(boolean speak, String dbms, String user, String pswd)
{
	try
	{
		Class.forName("com.mysql.jdbc.Driver");
    	Connection db = DriverManager.getConnection("jdbc:mysql://localhost/"+dbms,user,pswd);
    	pswd = "";
    	return db;
    }
    catch(Exception x)
    {
    	if(speak) p2putility.Util.logServer("Errore di connessione.\n"+x);
       	return null;
    }
}

/**
 * Disconnette la connessione specificata.
 *
 * @param speak attiva l'output di debug
 * @param db connessione da disconnettere
 * @return boolean se la disconnessione e' andata a buon fine
 */
public static boolean disconnect(boolean speak, Connection db)
{
	try
	{
		db.close();
		return true;
	}
	catch(Exception x)
	{
		if(speak) p2putility.Util.logServer("Errore nella chiusura della connessione.\n"+x);
		return false;
	}
}

/**
 * Crea uno Statement con la connessione specificata.
 *
 * @param speak attiva l'output di debug
 * @param db connessione da cui creare lo Statement
 * @return Statement con la connessione specificata
 */
public static Statement createStat(boolean speak, Connection db)
{
	try
	{
		return db.createStatement();
	}
	catch(Exception x)
	{
	 	if(speak) p2putility.Util.logServer("Errore nella creazione dello statement.\n"+x);
		return null;
	}
}

/**
 * Chiude lo Statement specificato.
 *
 * @param speak attiva l'output di debug
 * @param s Statement da chiudere
 * @return boolean se la chiusura e' andata a buon fine
 */
public static boolean closeStat(boolean speak, Statement s)
{
	try
	{
		s.close();
		return true;
	}
	catch(Exception x)
	{
	 	if(speak) p2putility.Util.logServer("Errore nella chiusura dello Statement.\n"+x);
		return false;
	}
}

/**
 * Esegue una query sullo Statement specificato.
 *
 * @param speak attiva l'output di debug
 * @param s Statement su cui eseguire la query
 * @param query query da eseguire
 * @return ResultSet risultato della query
 */
public static ResultSet yesRSetQuery(boolean speak, Statement s, String query)
{
	try
	{
		return s.executeQuery(query);
	}
	catch(Exception x)
	{
	 	if(speak) p2putility.Util.logServer("Errore in esecuzione query.\n"+x);
		return null;
	}
}

/**
 * Esegue una query sullo Statement specificato.
 *
 * @param speak attiva l'output di debug
 * @param s Statement su cui eseguire la query
 * @param query String query da eseguire
 * @return boolean true se il risultato della query e' nullo (e' corretto)
 */
public static boolean noRSetQuery(boolean speak, Statement s, String query)
{
	try
	{
		return !s.execute(query);
	}
	catch(Exception x)
	{
	 	if(speak) p2putility.Util.logServer("Errore in esecuzione query.\n"+x);
		return false;
	}
}

/**
 * Chiude il ResultSet specificato.
 *
 * @param speak attiva l'output di debug
 * @param r ResultSet da chiudere
 * @return boolean se la chiusura e' andata a buon fine
 */
public static boolean closeRSet(boolean speak, ResultSet r)
{
	try
	{
		r.close();
		return true;
	}
	catch(Exception x)
	{
	 	if(speak) p2putility.Util.logServer("Errore nella chiusura del ResultSet.\n"+x);
		return false;
	}
}

	
}