package p2pserver;

import java.sql.*;

/**
 * Classe che permette la preparazione del DBMS in modo da renderlo
 * accessibile in modo sicuro da un utente creato e non da ROOT.
 */
public class PrepareDB_root
{

    /**
     * Funzione per preparare il DBMS in modo da renderlo
     * accessibile in modo sicuro dall'utente creato.
     *
     * @param disperr debug dei messaggi di errore
     * @param dispok debug dei messaggi di successo
     * @param dbroot nome del DB a cui accedere da root (vedi MySQL)
     * @param userroot username dell'utente di root del DBMS
     * @param pswdroot password dell'utente di root del DBMS
     * @param userp2p username dell'utente da creare
     * @param  pswdp2p password dell'utente da creare
     * @param dbp2p nome del DB da creare
     * @return boolean true se e' andato tutto a buon fine
     * @see p2pserver.Connecttodbms
     */
	private static boolean prepare(	boolean disperr, boolean dispok, 
									String dbroot, String userroot, String pswdroot,
									String userp2p, String pswdp2p, String dbp2p
								  )
	{
		boolean ret = true;
		
		Connection db = p2pserver.Connecttodbms.connect(disperr,dbroot,userroot,pswdroot);
		pswdroot = "";
       	if(db == null) return false;
       	
        Statement s = p2pserver.Connecttodbms.createStat(disperr,db);
        if(s==null) return false;
                 
        boolean ok = false;
        String msg = "";
        
        
        ok = p2pserver.Connecttodbms.noRSetQuery(disperr,s,"drop database "+dbp2p+";");
        msg = "now";
        if(ok) msg = "already";
        if(dispok) p2putility.Util.logServer("Clean DB workspace "+msg+" ok.");
        
        
        ok = p2pserver.Connecttodbms.noRSetQuery(disperr,s,"drop user "+userp2p+"@localhost;");
        msg = "now";
        if(ok) msg = "already";
        if(dispok) p2putility.Util.logServer("Clean DB "+userp2p+" "+msg+" ok.");
        
        
        ok = p2pserver.Connecttodbms.noRSetQuery(disperr,s,"create database "+dbp2p+";");
        msg = "now";
        if(ok) msg = "already";
        else ret = false;
        
        if(dispok) p2putility.Util.logServer("Create DB "+dbp2p+" "+msg+" ok.");
        
        
        ok = p2pserver.Connecttodbms.noRSetQuery(disperr,s,"grant all on "+dbp2p+".* to "+
        										   userp2p+"@localhost IDENTIFIED BY '"+pswdp2p+"';");
        pswdp2p = "";
        msg = "now";
        if(ok) msg = "already";
        else ret = false;
        if(dispok) p2putility.Util.logServer("Create DB's user "+userp2p+" "+msg+" ok.");

                  
        p2pserver.Connecttodbms.closeStat(disperr,s);
        p2pserver.Connecttodbms.disconnect(disperr,db);
        
        return ret;
        
	}

    /**
     * Costruttore.
     * Richiama la funzione per preparare il DBMS e verifica l'output di quest'ultima.
     *
     * @param dispthis debug dei messaggi di questa funzione
     * @param dispallok debug dei messaggi di successo della funzione chiamata
     * @param dispallerr debug dei messaggi di errore della funzione chiamata
     * @param dbroot nome del DB a cui accedere da root (vedi MySQL)
     * @param userroot username dell'utente di root del DBMS
     * @param pswdroot password dell'utente di root del DBMS
     * @param userp2p username dell'utente da creare
     * @param pswdp2p password dell'utente da creare
     * @param dbp2p nome del DB da creare
     */
	public PrepareDB_root(	boolean dispthis, boolean dispallok, boolean dispallerr,
							String dbroot, String userroot, String pswdroot,
							String userp2p, String pswdp2p, String dbp2p
						 )
	{
		super();
        @SuppressWarnings("static-access")
		boolean ok = this.prepare(dispallerr,dispallok,dbroot,userroot,pswdroot,userp2p,pswdp2p,dbp2p);
		pswdp2p = "";
		pswdroot = "";
		
		if(dispthis)
			if(ok)	p2putility.Util.logServer("SETTING DataBase OK!");
			else	p2putility.Util.logServer("SETTING DataBase ERROR!");
			
	}
}