package p2pserver;

import java.sql.*;

/**
 * Classe che permette la preparazione delle tabelle accessibili dall'utente
 * specificato in configurazione, con password RANDOM.
 */
public class PrepareDB_user
{
	private static String tab1 = "idip";
	private static String tab2 = "files";

    /**
     * Restituisce il nome della tabella degli utenti
     * @return String nome tabella degli utenti
     */
	public static String tab1(){
		return tab1;
	}

    /**
     * Restituisce il nome della tabella dei file
     * @return String nome tabella dei file
     */
	public static String tab2(){
		return tab2;
	}
	
	/**
     * Crea le tabelle identificate da tab1 e tab2 secondo le specifiche adottate
     * dal programmatore e compatibili con il DBMS in uso.
     * @param dispok debug dei messaggi di successo
     * @param disperr debug dei messaggi di errore
     * @param userp2p username dell'utente di P2PFileSystem
     * @param pswdp2p password dell'utente di P2PFileSystem
     * @param dbp2p nome del DB a cui accedere
     * @return boolean true se e' andato tutto a buon fine
     * @see p2pserver.Connecttodbms
     */
	private static boolean prepare(	boolean dispok, boolean disperr,
									String userp2p, String pswdp2p, String dbp2p
								  )
	{
		boolean ret = true;
		
		Connection db = p2pserver.Connecttodbms.connect(disperr,dbp2p,userp2p,pswdp2p);
		pswdp2p = "";
       	if(db == null) return false;
       	
        Statement s = p2pserver.Connecttodbms.createStat(disperr,db);
        if(s==null) return false;
                 
        boolean ok = false;
        String msg = "";
        
        // tab1
        String query = "create table "+tab1+" ("+
        										"id int not null, "+
        										"ip char(16) not null, "+
        										"username varchar(30) unique not null, "+
        										"password varchar(40), "+
        										"primary key(id)"+
        									  ");";
        
        ok = p2pserver.Connecttodbms.noRSetQuery(disperr,s,query);
        if(!ok) ret = false;
        if(dispok) p2putility.Util.logServer("Create table "+tab1+" ok.");
        
        // tab2
        query = "create table "+tab2+
        		"("+
                "ckey bigint not null, "+
                "name varchar(1000) not null, "+
                "path mediumtext, "+
                "id int, "+
                "isdir boolean default false, "+
                "size bigint default 0, "+
                "cdate datetime, "+
                "rights int default 0, "+
                "state int default 0, "+
                "idate timestamp default current_timestamp on update current_timestamp, "+
                "primary key(ckey,id), "+
                "CONSTRAINT FOREIGN KEY (id) REFERENCES "+tab1+"(id) ON DELETE CASCADE ON UPDATE CASCADE "+
              	");";

        ok = p2pserver.Connecttodbms.noRSetQuery(disperr,s,query);
        if(!ok) ret = false;
        if(dispok) p2putility.Util.logServer("Create table "+tab2+" ok.");

       
        p2pserver.Connecttodbms.closeStat(disperr,s);
        p2pserver.Connecttodbms.disconnect(disperr,db);
        
        return ret;
        
	}

    /**
     * Costruttore.
     * Richiama la funzione per creare le tabelle e verifica l'output di quest'ultima.
     *
     * @param dispthis debug dei messaggi di questa funzione
     * @param dispallok debug dei messaggi di successo della funzione chiamata
     * @param dispallerr debug dei messaggi di errore della funzione chiamata
     * @param userp2p username dell'utente di P2PFileSystem
     * @param pswdp2p password dell'utente di P2PFileSystem
     * @param dbp2p nome del DB a cui accedere
     */
	public PrepareDB_user(	boolean dispthis, boolean dispallok, boolean dispallerr,
							String userp2p, String pswdp2p, String dbp2p
						 )
	{
		super();
        @SuppressWarnings("static-access")
		boolean ok = this.prepare(dispallok,dispallerr,userp2p,pswdp2p,dbp2p);
		pswdp2p = "";
		
		if(dispthis)
			if(ok) 	p2putility.Util.logServer("SETTING Tables OK!");
			else	p2putility.Util.logServer("SETTING Tables ERROR!");
	}
}