package p2pclient;

import java.rmi.Remote;
import java.util.ArrayList;

/***
 * Interfaccia delle funzioni RMI condivise dal server RMI "serverRMI"
 *
 * @see p2pserver.ServerRMI
 */
public interface RMIFuncConstructor extends Remote {

    /**
     * Interfaccia del metodo che disconnette un client dal server cancellandolo dal DBMS
     * @param usr username
     * @param pswd password
     * @throws java.rmi.RemoteException
     */
    public void disconnectClient(String usr, String pswd)throws java.rmi.RemoteException;

    /**
     * Interfaccia della funzione che ritorna true se il server e' attivo
     * @return boolean true se il server e' attivo
     * @throws java.rmi.RemoteException
     */
    boolean serverOn()throws java.rmi.RemoteException;

    /**
     * Interfaccia che verifica se il client e' connesso al server, cioe' se e' presente nel DBMS.
     * @param usr username
     * @param pswd password
     * @return boolean true se il client e' connesso al server
     * @throws java.rmi.RemoteException
     */
    boolean clientAlive(String usr, String pswd)throws java.rmi.RemoteException;

    /**
     * Interfaccia del metodo che costruisce la stringa di autenticazione
     * che permette ad un client registrato
     * di cancellare un file (se i diritti lo permettono) di un altro client
     * @param usr username
     * @param pswd password
     * @param owner_file username del possessore
     * @param id_file id univoco del file
     * @return String autorizzazione
     * @throws java.rmi.RemoteException
     */
    String getAccessStringDelete(String usr, String pswd, String owner_file, String id_file)throws java.rmi.RemoteException;

    /**
     * Interfaccia del metodo che costruisce la stringa di autenticazione che permette ad un client registrato
     * di scaricare un file (se i diritti lo permettono) di un altro client
     * @param usr username
     * @param pswd password
     * @param owner_file username del possessore
     * @param id_file id univoco del file
     * @return String autorizzazione
     * @throws java.rmi.RemoteException
     */
    String getAccessStringRead(String usr, String pswd, String owner_file, String id_file)throws java.rmi.RemoteException;

    /**
     * Interfaccia del metodo che costruisce la stringa di autenticazione
     * che permette ad un client registrato
     * di inviare un file (se i diritti lo permettono) di un altro client
     * @param usr username
     * @param pswd password
     * @param id_client2 username del possessore
     * @param id_file id univoco del file
     * @return String autorizzazione
     * @throws java.rmi.RemoteException
     */
    String getAccessStringUpload(String usr, String pswd, String id_client2, String id_file)throws java.rmi.RemoteException;

    /**
     * Interfaccia del metodo che restituisce la lista dei file presenti sul DBMS del server,
     * in base alla clausola WHERE specificabile in input.
     *
     * @param usr username
     * @param pswd password
     * @param  where (e' possiblie inserire codice SQL *)
     * @return ArrayList (ArrayList di ArrayList ) che contiene i risultati della query di getList
     * @throws java.rmi.RemoteException
     */
    ArrayList getList(String usr, String pswd, String where) throws java.rmi.RemoteException;

    /**
     * Interfaccia del metodo che restituisce la lista degli utenti presenti sul DBMS del server,
     * in base alla clausola WHERE specificabile in input.
     *
     * @param usr username
     * @param pswd password
     * @param where (e' possiblie inserire codice SQL *)
     * @return ArrayList (ArrayList di ArrayList ) che contiene i risultati della query di getUserList
     * @throws java.rmi.RemoteException
     */
    ArrayList getUserList(String usr, String pswd, String where) throws java.rmi.RemoteException;

    /**
     * Interfaccia del metodo che cancella i file presenti sul DBMS del server,
     * in base alla clausola WHERE specificabile in input.
     *
     * @param usr username
     * @param pswd password
     * @param name nome file
     * @param Path path del file
     * @param State stato del file
     * @return boolean true se la query e' andata a buon fine
     * @throws java.rmi.RemoteException
     */
    boolean removeFile(String usr, String pswd, String name, String Path, int State) throws java.rmi.RemoteException;
	
    /**
     * Interfaccia del metodo che invia una lista di file al DBMS del server.
     *
     * @param usr username
     * @param pswd password
     * @param list (ArrayList di ArrayList) che contiene i dati dei file da inviare
     * @return boolean true se la query e' andata a buon fine
     * @throws java.rmi.RemoteException
     */
    boolean sendList(String usr, String pswd, ArrayList list) throws java.rmi.RemoteException;
    
    /**
     * Interfaccia del metodo che inserisce un utente autorizzato (si veda p2putility.Util.login)
     * nella lista utenti del DBMS del server.
     *
     * @param user username
     * @param pswd password
     * @return int ID dell'utente generato dal server
     * @throws java.rmi.RemoteException
     */
    int registerToServer(String user, String pswd) throws java.rmi.RemoteException;
}
