package p2pserver;

import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;

/**
 * Classe che implementa un server in ascolto su una porta configurabile
 * in attesa di pacchetti UDP (qualsiasi). Invia al mandante del pacchetto
 * ricevuto un altro pacchetto che contiene l'indirizzo IP di questo server.
 * (Il pachetto inviato ha lunghezza 256 byte - i campi non occupati dall'IP sono null)
 */
public class ServerListener{
	
	public static boolean DBG = true;
	
    private static DatagramSocket socket;
    private static byte[] buf;
    private static DatagramPacket packet;


    /**
     * Costruttore.
     */
    public ServerListener(){}
    

    /**
     * [Vedi descrizione della classe]
     *
     * @param args { (int) porta sulla quale ascoltare, (boolean) output di debug }
     * @see p2pclient.ClientGetLocalServerList
     */
    public static void main(String[] args)
    {

        if(args.length<2)
        {
        	if(DBG) p2putility.Util.logServer("Missing argument/s: [port] [debug]");
        	return;
        }

        int lport = 0;
        try
        {
        	lport = Integer.parseInt(args[0]);
            DBG = Boolean.parseBoolean(args[1]);
        }
        catch(Exception e)
        {
        	if(DBG) p2putility.Util.logServer("Error in argument.");
        	return;
        }

    	try
    	{
	        socket = new DatagramSocket(lport); // (xxxx) porta a cui connettersi
	        buf = new byte[256];
	        packet = new DatagramPacket(buf, buf.length);
        }
        catch(Exception ex)
        {
        	if(DBG) p2putility.Util.logServer("SERVER error!"+ex);
        	return;
        }
        
        if(DBG) p2putility.Util.logServer("Server started...");
        
        while(true)
        {
        	try
        	{
                socket.receive(packet);
                     	
                InetAddress address = InetAddress.getByName(new String(packet.getData()));
                
				int port = packet.getPort();
				
				if(DBG) p2putility.Util.logServer("Request from "+address+" port "+port);
                
                buf = InetAddress.getLocalHost().getHostAddress().getBytes();
                
				socket.send(new DatagramPacket(buf, buf.length, address, port));	

            }
            catch (Exception ex){
                if(DBG) p2putility.Util.logServer("Server (UDP) listener error:"+ex);
                /* Show must go on */
            }
        }
    }
}
