package p2pserver;

import p2pclient.RMIFuncConstructor;
import java.rmi.*;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;
import java.util.ArrayList;


/*
 * Classe che implementa funzioni RMI, ovvero tutte le funzioni
 * accessibili da client remoti 'autorizzati' cioe' a conoscenza
 * dell'indirizzo IP, del protocollo (RMI), del nome assegnato
 * dal programmatore (ServerRMI) e dalla porta.
 *
 * Si precisa che tutte le funzioni ad eccezione di serverOn(),
 * che altro non fa che ritornare un valore true se il server e' attivo,
 * sono protette da un meccanismo di autenticazione user,password che
 * impedisce l'accesso a dati potenzialmente sensibili da parte di
 * client non autorizzati espressamente dal server.
 * @see p2pclient.RMIFuncConstructor
 * @see p2pclient.clientRMI
 */
public class ServerRMI extends UnicastRemoteObject implements RMIFuncConstructor {

    static ServerRMI obj;
    boolean dbg = true;
    private String user;
    private int id_user;

    /**
     * Costruttore.
     * @throws RemoteException
     */
	protected ServerRMI() throws RemoteException {
		super();

	}

    /**
     * Disconnette un client dal server cancellandolo dal DBMS
     * @param usr username
     * @param pswd password
     */
    public void disconnectClient(String usr, String pswd)throws java.rmi.RemoteException {
        ArrayList u = p2pgui.ServerFrame.IDB.getUserList("WHERE username='"+usr+"'");
        String id_client = ((ArrayList)u.get(0)).get(0).toString();

        p2pgui.ServerFrame.IDB.delFileListBy(null, null, null, id_client, null, null, null, null, null, null);
        p2pgui.ServerFrame.IDB.delUserListBy(id_client, null, usr, pswd);

        if(dbg) p2putility.Util.logServer("Information: Client "+id_client+" is disconnect.");

    }

    /**
     * Ritorna true se il server e' attivo
     * @return boolean true se il server e' attivo
     */
    public boolean serverOn()throws java.rmi.RemoteException{
        if(p2pgui.ServerFrame.IDB.isOpen()){
            if(dbg) p2putility.Util.logServer("SERVER SIDE: ON! ");
            return true;
        }

        if(dbg) p2putility.Util.logServer("SERVER SIDE: DOWN! ");
        return false;
    }

    /**
     * Verifica se il client e' connesso al server, cioe' se e' presente nel DBMS.
     * @param usr username
     * @param pswd password
     * @return boolean true se il client e' connesso al server
     */
     public boolean clientAlive(String usr, String pswd)throws java.rmi.RemoteException{
        if(!isReg(usr,pswd)){
            if(dbg) p2putility.Util.logServer("Client "+usr+" is not connected.");
            return false;
        }

        if(dbg) p2putility.Util.logServer("Client "+usr+" is connected.");
        return true;
    }

     /**
     * Costruisce la stringa di autenticazione che permette ad un client registrato
     * di cancellare un file (se i diritti lo permettono) di un altro client
     *
     * @param usr username
     * @param pswd password
     * @param owner_file username del possessore
     * @param id_file id univoco del file
     * @return String autorizzazione
     * @see p2pclient.C2C_server
     */
    @SuppressWarnings("static-access")
    public String getAccessStringDelete(String usr, String pswd, String owner_file, String id_file)throws java.rmi.RemoteException{
        if(!isReg(usr,pswd)){
            if(dbg) p2putility.Util.logServer("Client "+usr+" not registred on server.");
            return "-2";
        }
        String access = "";

        //ArrayList del file selezionato
        ArrayList l = p2pgui.ServerFrame.IDB.getFileList("WHERE ckey ="+id_file);
        //ArrayList dell'owner del file selezionato
        ArrayList i = p2pgui.ServerFrame.IDB.getUserList("WHERE id ="+owner_file);
        if(i.size()==1 && l.size()==1){
            ArrayList ll = (ArrayList) l.get(0);
            ArrayList ii = (ArrayList) i.get(0);

            int state = Integer.parseInt(ll.get(8).toString());
            int file_right = Integer.parseInt(ll.get(7).toString());
            if(file_right<2){
                p2putility.Util.logServer("Access  deny: no rights to delete the file");
                return "-4";
            }
            else {
                if(state==2){
                    p2putility.Util.logServer("Impossible to delete a file in writing.");
                    return "-3";
                }
                else
                    if(state==1){
                    p2putility.Util.logServer("Impossible to delete a file in reading.");
                    return "-5";
                }

                String ip_owner = ii.get(1).toString();
                String pswd_owner = ii.get(3).toString();
                String user_owner = ii.get(2).toString();

                access = pswd + pswd_owner + id_file;
                String v = p2putility.Util.hash_sha(access);
                access = ip_owner + ";" + v;

                 //Thread per inviare i dati d'accesso al client che possiede il file
                ThreadRmiAuthorization TRA = new ThreadRmiAuthorization();
                TRA.dbg = true;
                TRA.k = usr + id_file;
                TRA.v = v;
                TRA.ip = ip_owner;
                TRA.funct = "addAuthUser";
                ClientRmiAuthorization.user_to_connect = user_owner;
                ClientRmiAuthorization.pswd_to_connect = pswd_owner;

                Thread t = new Thread(TRA);
                t.start();
            }
        }

        return access;
    }

    /**
     * Costruisce la stringa di autenticazione che permette ad un client registrato
     * di scaricare un file (se i diritti lo permettono) di un altro client
     *
     * @param  usr username
     * @param pswd password
     * @param owner_file username del possessore
     * @param id_file id univoco del file
     * @return String autorizzazione
     * @see p2pclient.C2C_server
     */
    @SuppressWarnings("static-access")
    public String getAccessStringRead(String usr, String pswd, String owner_file, String id_file)throws java.rmi.RemoteException{
        if(!isReg(usr,pswd)){
            if(dbg) p2putility.Util.logServer("Client "+usr+" not registred on server.");
            return "-2";
        }
        String access = "";

        String ip_owner = "";
        String user_owner = "";
        String pswd_owner= "";

        //ArrayList del file selezionato
        ArrayList l = p2pgui.ServerFrame.IDB.getFileList("WHERE ckey ="+id_file);
        //ArrayList dell'owner del file selezionato
        ArrayList i = p2pgui.ServerFrame.IDB.getUserList("WHERE id ="+owner_file);
        if(i.size()==1 && l.size()==1){
            ArrayList ll = (ArrayList) l.get(0);
            ArrayList ii = (ArrayList) i.get(0);

            int state = Integer.parseInt(ll.get(8).toString());
            int file_right = Integer.parseInt(ll.get(7).toString());
            if(file_right<1){
                if(dbg) p2putility.Util.logServer("Access deny: no rights to open the file");
                return "-4";
            }
            else {
                if(state==2){
                    if(dbg) p2putility.Util.logServer("Impossible to open a file in writing.");
                    return "-3";
                }

                ip_owner = ii.get(1).toString();
                pswd_owner = ii.get(3).toString();

                user_owner = ii.get(2).toString();

                access = pswd + pswd_owner + id_file;
                String v = p2putility.Util.hash_sha(access);
                access = ip_owner + ";" + v;

                //Thread per inviare i dati d'accesso al client che possiede il file
                ThreadRmiAuthorization TRA = new ThreadRmiAuthorization();
                TRA.dbg = true;
                TRA.v = v;
                TRA.k = usr + id_file;
                TRA.ip = ip_owner;
                TRA.funct = "addAuthUser";
                ClientRmiAuthorization.user_to_connect = user_owner;
                ClientRmiAuthorization.pswd_to_connect = pswd_owner;

                Thread t = new Thread(TRA);
                t.start();
            }
        }

        return access;
    }

    /**
     * Costruisce la stringa di autenticazione che permette ad un client registrato
     * di inviare un file (se i diritti lo permettono) di un altro client
     *
     * @param usr username
     * @param pswd password
     * @param id_client2 username del possessore
     * @param id_file id univoco del file
     * @return String autorizzazione
     * @see p2pclient.C2C_server
     */
    @SuppressWarnings("static-access")
     public String getAccessStringUpload(String usr, String pswd, String id_client2, String id_file)throws java.rmi.RemoteException{
        if(!isReg(usr,pswd)){
            if(dbg) p2putility.Util.logServer("Client "+usr+" not registred on server.");
            return "-2";
        }
        String access = "";

        String ip_client2 = "";
        String user_client2 = "";
        String pswd_client2= "";

        //ArrayList dell'owner del file selezionato
        ArrayList i = p2pgui.ServerFrame.IDB.getUserList("WHERE id ="+id_client2);

        if(i.size()!=1)
            return "";

        ArrayList ii = (ArrayList) i.get(0);

        ip_client2 = ii.get(1).toString();
        pswd_client2 = ii.get(3).toString();
        user_client2 = ii.get(2).toString();

        access = pswd + pswd_client2 + id_file;
        access= p2putility.Util.hash_sha(access);

        //Thread per inviare i dati d'accesso al client che possiede il file
        ThreadRmiAuthorization TRA = new ThreadRmiAuthorization();
        TRA.dbg = true;
        TRA.k = usr + p2putility.Util.hash_sha(id_file);
        TRA.v = access;
        TRA.ip = ip_client2;
        TRA.funct = "addAuthUser";
        ClientRmiAuthorization.user_to_connect = user_client2;
        ClientRmiAuthorization.pswd_to_connect = pswd_client2;

        Thread t = new Thread(TRA);
        t.start();

        return access;
    }

    /**
     * Restituisce la lista dei file presenti sul DBMS del server,
     * in base alla clausola WHERE specificabile in input.
     *
     * @param usr username
     * @param pswd password
     * @param where (e' possiblie inserire codice SQL *)
     * @return ArrayList (ArrayList di ArrayList ) che contiene i risultati della query di getList
     */
    public ArrayList getList(String usr, String pswd, String where) throws java.rmi.RemoteException{
         if(!isReg(usr,pswd)){
            if(dbg) p2putility.Util.logServer("Client "+usr+" not registred on server.");
            return null;
        }

        try{
            return p2pgui.ServerFrame.IDB.getFileList(where);
        }
        catch(Exception xe){return null;}
    }

    /**
     * Restituisce la lista degli utenti presenti sul DBMS del server,
     * in base alla clausola WHERE specificabile in input.
     *
     * @param usr username
     * @param pswd password
     * @param where (e' possiblie inserire codice SQL *)
     * @return ArrayList (ArrayList di ArrayList ) che contiene i risultati della query di getUserList
     */
    public ArrayList getUserList(String usr, String pswd, String where) throws java.rmi.RemoteException{
         if(!isReg(usr,pswd)){
            if(dbg) p2putility.Util.logServer("Client "+usr+" not registred on server.");
            return null;
        }

        try{
            return p2pgui.ServerFrame.IDB.getUserList(where);
        }
        catch(Exception xe){
            if(dbg) p2putility.Util.logServer("getUserList error: "+xe);
            return null;
        }
    }

    /**
     * Cancella i file presenti sul DBMS del server,
     * in base alla clausola WHERE specificabile in input.
     *
     * @param usr username
     * @param pswd password
     * @param name nome file
     * @param path path file
     * @param state stato del file
     * @return boolean true se la query e' andata a buon fine
     */
    public boolean removeFile(String usr, String pswd, String name, String path, int state) throws java.rmi.RemoteException{
         if(!isReg(usr,pswd)){
            if(dbg) p2putility.Util.logServer("Client "+usr+" not registred on server.");
            return false;
        }

        int stato = 0;
        return p2pgui.ServerFrame.IDB.delFileListBy(null, name, path, null, null, null, null, null, ""+stato , null);
       
    }

    /**
     * Invia una lista di file al DBMS del server.
     *
     * @param usr username
     * @param pswd password
     * @param list ArrayList (ArrayList di ArrayList) che contiene i dati dei file da inviare
     * @return boolean true se la query e' andata a buon fine
     */
    public boolean sendList(String usr, String pswd, ArrayList list) throws java.rmi.RemoteException{
         if(!isReg(usr,pswd)){
            if(dbg) p2putility.Util.logServer("Client "+usr+" not registred on server.");
            return false;
        }

        if(dbg) p2putility.Util.logServer("Server: "+list);
        //Query x trovare numero di file nella tabella x assegare id del file
        long next = p2pgui.ServerFrame.IDB.nextCkeyFileList();
        for(int i=0; i<list.size(); i++){
            ((ArrayList) list.get(i)).remove(0);
            ((ArrayList) list.get(i)).add(0, next+i);
        }
        p2pgui.ServerFrame.IDB.putFileList(list);
        return true;
    }

   
    /**
     * Inserisce un utente autorizzato (si veda p2putility.Util.login)
     * nella lista utenti del DBMS del server.
     *
     * @param usr username
     * @param pswd password
     * @return int ID dell'utente generato dal server
     */
    @SuppressWarnings("static-access")
    public int registerToServer(String usr, String pswd) throws java.rmi.RemoteException{
        user = usr;
        if(!p2putility.Util.login(user, pswd)){
            if(dbg) p2putility.Util.logServer("ID or PASSWORD not valid");
            return -1;
        }     

        int nextid = 0;
        String ip = "";
        try{
            ip = obj.getClientHost();
        }
		catch(Exception x){}
        ArrayList tmp2 = new ArrayList();
        ArrayList tmp1 = new ArrayList();
        
        nextid = p2pgui.ServerFrame.IDB.nextUserID();
        
        tmp1.add(nextid);
        tmp1.add(ip);
        tmp1.add(user);
        tmp1.add(pswd);
        tmp2.add(0,tmp1);

        if(dbg) p2putility.Util.logServer("Arraylist da inserire: "+tmp2);
        boolean result = p2pgui.ServerFrame.IDB.putUserList(tmp2);
        if(dbg) p2putility.Util.logServer("Risultato della query di inserimento "+result);

        if (!result)
            return -1;

        ArrayList id = p2pgui.ServerFrame.IDB.getUserList("WHERE username='"+user+"'");
        ArrayList idd = (ArrayList)id.get(0);
        id_user = Integer.parseInt(idd.get(0).toString());

        return id_user;
    }



	/**
     * Main. Avvia il server RMI.
	 * @param  args (inutilizzato)
	 */
	public static void main(String[] args) {
		
		try {
			obj = new ServerRMI();
            Registry registry;

            if(!p2putility.Config.r_clientOpen())
                registry = LocateRegistry.createRegistry(p2putility.Config.r_rmiregport());
            else
                registry = LocateRegistry.getRegistry();

            registry.rebind("ServerRMI",obj);
            Naming.rebind("ServerRMI", obj);
          

			p2putility.Util.logServer("ServerRMI e' stato registrato");
		} catch (Exception e) {
			p2putility.Util.logServer("ServerRMI err: " + e);
		}
		
	}

    /**
     * Funzione privata che verifica se un utente e' registrato nel DBMS del server.
     * @param u username
     * @param p password
     * @return boolean true se e' registrato e il server puo' verificarlo, false altrimenti
     */
    private boolean isReg(String u, String p){
        try
        {
            ArrayList l = p2pgui.ServerFrame.IDB.getUserList("WHERE username='"+u+"'");
            p2putility.Util.logServer("isReg("+u+","+p+") = "+l);
            if(l.size()>0)
                if(((ArrayList)l.get(0)).get(3).toString().compareTo(p)==0)
                   return true;
        }
        catch(Exception xe){
            if(dbg) p2putility.Util.logServer("isReg error:"+xe);
            return false;
        }
        return false;
    }
    
}
