package p2putility;

import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.security.MessageDigest;

/**
 * Classe che implementa metodi vari di utilita' generale
 */
public class Util {

    /**
     * Costruttore
     */
    public Util(){
        super();
    }

    /**
     * Verifica se un utente fa parte della lista di utenti registrati
     * (Il programma P2PFileSystem non si occupa della sua gestione)
     *
     * @param u Username
     * @param p Password
     * @return boolean restituisce true se l'utente e' registrato, false altrimenti
     */
    public static boolean login(String u, String p) {

        File f = new File(  p2putility.Config.r_currentpath()   +
                            p2putility.Config.r_configpath()    +
                            p2putility.Config.r_userlistfile()
                         );
        
		if(f.exists()){
			try{
				DataInputStream d = new DataInputStream(new FileInputStream(f));

                /** Struttura del file "userlistfile" - "user","password"  - stile CSV */
                String v = "\""+u+"\",\""+p+"\"";

                if(d.available()>0) d.readLine(); // Salto la prima riga (Struttura CSV)
                
                while(d.available()>0){
                    String c = d.readLine();
                    if(c.startsWith(v)){
                        d.close();
                        return true;
                    }
                }
				
			}
			catch(IOException e){return false;}
		}
        return false;
    }

    /**
     * Converte un array di byte in una stringa esadecimale
     *
     * @param data [] dati
     * @return String dati in forma esadecimale
     */
    private static String convertToHex(byte[] data)
    {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < data.length; i++) {
            int halfbyte = (data[i] >>> 4) & 0x0F;
            int two_halfs = 0;
            do {
                if ((0 <= halfbyte) && (halfbyte <= 9))
                    buf.append((char) ('0' + halfbyte));
                else
                    buf.append((char) ('a' + (halfbyte - 10)));
                halfbyte = data[i] & 0x0F;
            } while(two_halfs++ < 1);
        }
        return buf.toString();
    }

    /**
     * Converte una stringa nel corrispondente HASH SHA-1 in forma esadecimale
     * @param text dati
     * @return String Hash SHA-1 dei dati in forma esadecimale
     */
    public static String hash_sha(String text)
    {
        try
        {
            MessageDigest md;
            md = MessageDigest.getInstance("SHA-1");
            byte[] sha1hash = new byte[40];
            md.update(text.getBytes("iso-8859-1"), 0, text.length());
            sha1hash = md.digest();
            return convertToHex(sha1hash);
        }
        catch(Exception he){
            p2putility.Util.logGeneric("UTIL: hash_sha error: "+he);
            return "";
        }
    }

    /**
     * Scrive i dati in input sul file di log del Server (se questo esiste valido)
     * @param data dati
     */
     public static void logServer(String data){
        if(p2putility.Config.r_serverlog().length()<2) // no debug file active
            return;

        writeLogOn(p2putility.Config.r_serverlog(),data);
    }

    /**
     * Scrive i dati in input sul file di log del Client (se questo esiste valido)
     * @param data dati
     */
    public static void logClient(String data){
        if(p2putility.Config.r_clientlog().length()<2) // no debug file active
            return;

        writeLogOn(p2putility.Config.r_clientlog(),data);
    }

    /**
     * Scrive i dati in input sul file di log Generico (se questo esiste valido)
     * @param data dati
     */
    public static void logGeneric(String data){
        if(p2putility.Config.r_genericlog().length()<2) // no debug file active
            return;

        writeLogOn(p2putility.Config.r_genericlog(),data);
    }

    /**
     * Scrive i dati in input sul file passato come parametro
     * @param f nome del file
     * @param data dati
     */
    private static void writeLogOn(String f, String data){
        File ff = new File(f);
        try{
            if(ff.exists()){
                ff.setReadable(true);
                ff.setWritable(true);
            }
        }catch(Exception e){}
        try{
            FileWriter fr = new FileWriter(f,ff.exists());
            fr.write("\n***** "+new java.util.Date()+" *****\n"+data+"\n");
            fr.close();
        }catch(Exception e){
            javax.swing.JOptionPane.showMessageDialog(null,"Error in LOGGING... \n"+e,"Error",0);
        }
    }
}
